#ifndef JAGOBJ_H
#define JAGOBJ_H

struct ObjBitmap { /* bitmap object: unpacked version */
   unsigned short type;
   unsigned short xpos;
   unsigned short ypos;
   unsigned long link;   /* if 0, fall through to next object */
   void	*data;
   unsigned short height;
   unsigned short dwidth;
   unsigned short iwidth;
   unsigned short depth;
   unsigned short pitch;
   unsigned short index; /* palette index */
   unsigned short flags;
#define OL_REFLECT 0x1
#define OL_RMW     0x2
#define OL_TRANS   0x4
#define OL_RELEASE 0x8
   unsigned short firstpix;
   unsigned short hscale;
   unsigned short vscale;
   unsigned short remainder;
};

struct ObjGpu {
   unsigned short type;
   unsigned short reserved;
   unsigned short ypos;
   unsigned long data[2];
};

struct ObjBranch {
   unsigned short type;
   unsigned short condition;
#define OL_CCEQ  0x1
#define OL_CCGT  0x2
#define OL_CCLT  0x4
#define OL_CCFLG 0x8
#define OL_CC2ND 0x10
   unsigned short ypos;
   unsigned long link;
};

struct ObjStop {
   unsigned short type;
   unsigned short reserved;
   unsigned short intflag;
#define OL_INT 1
   unsigned long data[2];
};

union ObjList {
   struct ObjBitmap bit;
   struct ObjGpu    gpu;
   struct ObjBranch bra;
   struct ObjStop   stp;
   unsigned short type;
};

/* defines for "type" of olist union */
#define OL_BITMAP      0
#define OL_SCALEBITMAP 1
#define OL_GPU         2
#define OL_BRANCH      3
#define OL_STOP        4
#define OL_SKIP        0xfe

/* defines for various scaling factors */
#define OL_SCALE_NONE   0x20
#define OL_SCALE_HALF   0x10
#define OL_SCALE_DOUBLE 0x40
#define OL_SCALE_TRIPLE 0x60

typedef struct {
   long high_long;
   long low_long;
} ObjPhrase;

typedef struct {
   ObjPhrase *bitmap_obj;
   ObjPhrase first_phrase;
} ObjBitmapBackupEntry;

void ObjSetOlp(void *obj_list, long backup_entries, void *bitmap_backup);
void ObjInitVbint(void);
void ObjCreateBitmap(ObjPhrase *obj, unsigned int ypos,
                     unsigned int height, ObjPhrase *next_addr, void *data,
                     unsigned int xpos, unsigned long depth,
                     unsigned long pitch, unsigned int dwidth,
                     unsigned int iwidth, unsigned int index,
                     unsigned long reflect, unsigned long rmw,
                     unsigned long trans, unsigned long release,
                     unsigned int firstpix);
void OjCreateBranch(ObjPhrase *obj, unsigned int ypos, unsigned long cc,
                    ObjPhrase *next_addr);
void ObjCreateStop(ObjPhrase *obj, unsigned long ints);
void ObjBackupBitmap(ObjBitmapBackupEntry *backup, ObjPhrase *obj);
ObjPhrase *ObjBuild(union ObjList *unpacked);
void ObjBldTo(union ObjList *unpacked, ObjPhrase *ptr);
long ObjSize(union ObjList *unpacked);

#endif
